uniform sampler2D texture0; // color texture
uniform sampler2D texture1; // depth texture
uniform sampler2D texture2; // normal texture
uniform sampler2D texture3; // noise texture

//uniform vec2 viewPort2;			// viewPort dimensions
uniform vec4 viewFrustum;		// x: view angle (radians); y: ratio (y / x); z: zNear; w: zFar

uniform float radius;
uniform int renderMode;

varying vec3 cameraRay;
varying vec2 invNearPlane;

#define KERNEL_SIZE 10	
const vec3 kernel[KERNEL_SIZE] = {

	vec3(1.0, 0.0, 0.0),
	vec3(0.0, 1.0, 0.0),
	vec3(0.0, 0.0, 1.0),
	
	vec3(0.707107, 0.707107, 0.0),
	vec3(0.707107, 0.0, 0.707107),
	vec3(0.0, 0.707107, 0.707107),
	
	vec3(0.577350, 0.577350, 0.577350),	
	vec3(-0.577350, 0.577350, 0.577350),
	vec3(0.577350, -0.577350, 0.577350),
	vec3(0.577350, 0.577350, -0.577350)
	
};


void main() {

	if (renderMode == 2) {
		// Show texture only
		gl_FragColor = texture2D(texture0, gl_TexCoord[0].st);
		return;
	} else if (renderMode == 5) {
		// Show depth
		gl_FragColor = texture2D(texture1, gl_TexCoord[0].st);
		return;
	} else if (renderMode == 6) {
		// Show normals
		gl_FragColor = texture2D(texture2, gl_TexCoord[0].st);
		return;
	} else if (renderMode == 7) {
		// Show noise
		gl_FragColor = texture2D(texture3, gl_TexCoord[0].st * 4.0);
		return;
	}

	float z = texture2D(texture1, gl_TexCoord[0].st).x;
	// Skip pixels which are the sky
	if (z >= 1.0) {
		gl_FragColor = texture2D(texture0, gl_TexCoord[0].st);
		return;
	}
	
	vec3 normal = texture2D(texture2, gl_TexCoord[0].st).xyz;
	// Need to transform the normal from [0, 1] to [-1, 1]
	normal = normalize(normal * 2.0 - 1.0);
	
	// Expand the point on the near plane according to its depth
	vec3 pointRay = cameraRay * (1.0 + z * (viewFrustum.w - viewFrustum.z));
	// Get a random vector for each pixel
	vec3 randomVector = normalize(texture2D(texture3, gl_TexCoord[0].st * 4.0).xyz);
	
	int samples = 10;	
	float occlusion = 0.0;
	
	for(int i = 0; i < samples; i++) {
	
		// Rotate the kernel for each pixel
		vec3 samplingRay = reflect(kernel[i], randomVector) * radius;
		
		// If the ray points into the object, flip it
		if (dot(samplingRay, normal) < 0.0) {
			samplingRay *= -1.0;
		}
		
		vec3 samplingPoint = pointRay + samplingRay;
		
		// Get screen coordinate of the sample point
		// First reduce the vector so that its z value is at the near clipping plane
		vec2 screenSamplingPoint = ((samplingPoint / samplingPoint.z) * viewFrustum.z).xy;
		// Then change its X and Y range to [0, 1] again
		screenSamplingPoint = screenSamplingPoint * invNearPlane + 0.5;
		
		// Maybe skip pixels outside of screen
		
		// Get the z value of the sampling point
		float sampleZ = texture2D(texture1, screenSamplingPoint).x;
		
		// No need to skip sampleZ >= 1.0, because z can't be >= 1.0 and would fail the next test anyway
		
		// If sampleZ is smaller than z, the ray got occluded
		if (sampleZ < z) {
			occlusion += (1.0 / float(samples));
		}
	}
	
	if (renderMode == 3) {
		// Show ssao only
		gl_FragColor = vec4(1.0 - occlusion);
		return;
	} else if (renderMode == 4) {
		// Show split screen
		
		gl_FragColor = texture2D(texture0, gl_TexCoord[0].st) * (1.0 - occlusion * (gl_TexCoord[0].s < 0.5 ? 1.0 : 0.0));
		
		return;
	}
	
	gl_FragColor = texture2D(texture0, gl_TexCoord[0].st) * (1.0 - occlusion);

}
